##
## Code to integrate CVS interaction
## within the SDM
##
## ben@mit.edu

proc cvs_dir {} {
    return "/web/acstest/sdm-cvs"
}

proc cvs_executable {} {
    return "/usr/bin/cvs"
}

proc cvs {path {hostname ""} {username ""} {password ""}} {
    return "[cvs_executable] -d $path"
}

proc cvs_checkout {package_name path {release_version ""} {hostname ""} {username ""} {password ""}} {
    if {$release_version == ""} {
	set release_cvs "-A -d"
    } else {
	set release_cvs "-d -r $release_version"
    }

    if {[catch {eval "exec [cvs $path $hostname $username $password] checkout $package_name"} errmsg]} {
	ns_log Notice "cvserror: $errmsg"
    }

    if {[catch {eval "exec [cvs $path $hostname $username $password] update $release_cvs $package_name"} errmsg]} {
	ns_log Notice "cvserror: $errmsg"
    }
}

proc cvs_commit {package_name path commit_msg {hostname ""} {username ""} {password ""}} {
    if {[catch {eval "exec [cvs $path $hostname $username $password] commit -m \"$commit_msg\""} errmsg]} {
	ns_log Notice "cvserror: $errmsg"
    }
}

proc cvs_tag_to_cli {tag} {
    if {$tag == ""} {
	return "-A"
    } else {
	return "-r $tag"
    }
}    

proc cvs_checkout_to_string {file_name cvsroot tag} {
    
    if {[catch {eval "exec [cvs $cvsroot] checkout [cvs_tag_to_cli $tag] -p $file_name"} errmsg]} {
	set stuff $errmsg
    } else {
	return ""
    }

    set start [string first "==================================================================" $stuff]
    set end [string first "\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*" [string range $stuff [expr $start + 1] end]]
    if {$end > -1} {
	set end [expr $start + $end]
    }

    incr start -1

    ns_log Notice "start= $start; end= $end"

    set file_contents [string range $stuff 0 $start]
    append file_contents [string range $stuff [expr "$end + 15"] end]

    return $file_contents
}

## History Abstraction

proc cvs_history_create {revision date author comments} {
    return [list $revision $date $author $comments]
}

proc cvs_history_get_revision {hist} {
    return [lindex $hist 0]
}

proc cvs_history_get_date {hist} {
    return [lindex $hist 1]
}

proc cvs_history_get_author {hist} {
    return [lindex $hist 2]
}

proc cvs_history_get_comments {hist} {
    return [lindex $hist 3]
}

proc cvs_get_cvsroot {package_id db} {
    set selection [ns_db 0or1row $db "select cvs_path from cvs_package_data where package_id=$package_id"]
    
    if {$selection == ""} {
	return ""
    }

    set_variables_after_query

    return $cvs_path
}


# Returns a list of history items
proc cvs_file_get_history {dir file_name cvsroot} {
    set list_of_history [list]

    # Get the CVS history 
    cd $dir
    if {[catch {set stuff [eval "exec [cvs $cvsroot] log $file_name"]} errmsg]} {
	ns_log Notice "error: $errmsg"
	return ""
    } else {
    }
    
    set start_index 0

    while {1} {
	if {$start_index < 0} {
	    break
	}

	set blob_start $start_index
	set blob_end [string first "----------------------------" [string range $stuff $blob_start end]]
	if {$blob_end > -1} {
	    set blob_end [expr $blob_start + $blob_end]
	}

	ns_log Notice "$blob_start - $blob_end"

	if {$blob_end == -1} {
	    set blob_end [string first "============" [string range $stuff $blob_start end]]
	    if {$blob_end > -1} {
		set blob_end [expr $blob_start + $blob_end]
	    }

	    set start_index -1
	} else {
	    set start_index [expr "$blob_end + 28"]
	}

	set blob [string range $stuff $blob_start [expr "$blob_end - 1"]]

	ns_log Notice "one blob: $blob"
	
	if {![regexp {revision ([0-9\.]*)} $blob all revision]} {
	    continue
	}

	if {![regexp {date: ([^;]*);} $blob all date]} {
	    continue
	}
	
	if {![regexp {author: ([^;]*);} $blob all author]} {
	    continue
	}
	
	# Find the comments, this SUCKS! CVS does not provide nicely parsable stuff.
	set last_colon [string last ":" $blob]
	set start_comment [string first "\n" [string range $blob $last_colon end]]

	ns_log Notice "last_colon: $last_colon, $start_comment"
	set comments [string range $blob [expr $last_colon + $start_comment] end]
	ns_log Notice "comment: $comments"

	lappend list_of_history [cvs_history_create $revision $date $author $comments]
    }

    return $list_of_history
}


proc cvs_util_split_by_lines {content} {
    return [split $content "\n"]
}

proc cvs_util_line_range {split_file start_line end_line} {
    set return_stuff ""

    set start [expr "$start_line - 1"]
    set end [expr "$end_line - 1"]

    for {set counter $start} {$counter <= $end} {incr counter} {
	append return_stuff [lindex $split_file $counter] "\n"
    }

    return $return_stuff
}

proc cvs_file_diff {dir file_name cvsroot old_release new_release} {

    # Get the CVS diff
    cd $dir
    if {[catch {eval "exec [cvs $cvsroot] diff -r $old_release -r $new_release $file_name"} errmsg]} {
	set stuff $errmsg
	# ns_log Notice "$stuff"
    } else {
	return ""
    }

    # Get the latest version of the file for full diff
    set latest_version [cvs_checkout_to_string $file_name $cvsroot $new_release]

    # Split by lines
    set split_latest [cvs_util_split_by_lines $latest_version]

    set start_of_diff [string first "diff -r" $stuff]
    set start_of_useful_stuff [expr "[string first "\n" [string range $stuff $start_of_diff end]] + 1 + $start_of_diff"]
    set useful_diff [string range $stuff $start_of_useful_stuff end]

    # Now create the diff info
    set line_counter 1
    set diff_content [list]
    set split_diff [cvs_util_split_by_lines $useful_diff]
    set current_section ""

    foreach split_line $split_diff {
	# If we're at the beginning/end of a section
	if {[regexp {^([0-9,]*)([adc])([0-9,]*)$} $split_line all old_lines action new_lines]} {

	    # Wrap up the previous section
	    if {$current_section != ""} {
		lappend diff_content [cvs_process_one_section $current_section $current_action]
		
		set current_section ""
	    }

	    # In case there are multiple lines
	    if {![regexp {([0-9]*),([0-9]*)} $new_lines all new_lines_start new_lines_end]} {
		set new_lines_start $new_lines
		set new_lines_end $new_lines
	    }
	    
	    # Add a section to the diff we skipped some stuff
	    if {$new_lines_start > $line_counter} {
		lappend diff_content [list "nochange" [cvs_util_line_range $split_latest [expr "$line_counter + 1"] [expr "$new_lines_start - 1"]]]
	    }

	    set line_counter $new_lines_end
	    
	    set current_action $action
	    continue
	}

	# This is just another line, add it to the section
	ns_log Notice "one CVS diff line: $split_line"
	set first_two [string range $split_line 0 1]
	if {$first_two == "< " || $first_two == "> "} {
	    append current_section [string range $split_line 2 end] "\n"
	} else {
	    if {$first_two == "\\ " || $first_two == "--"} {
		append current_section $split_line "\n"
	    }
	}

    }

    # Plop the last section down
    if {$current_section != ""} {
	lappend diff_content [cvs_process_one_section $current_section $current_action]
    }

    # Add the end if necessary
    if {$line_counter < [llength $split_latest]} {
	lappend diff_content [list "nochange" [cvs_util_line_range $split_latest [expr "$line_counter + 1"] [llength $split_latest]]]
    }

    return $diff_content
}


proc cvs_process_one_section {current_section current_action} {
    if {$current_action == "c"} {
	# split the stuff along the "---"
	set split_location [string first "\n---\n" $current_section]
	
	# add the stuff
	return [list "change" "[string range $current_section 0 $split_location]\n" [string range $current_section [expr "$split_location + 5"] end]]
    }
    
    if {$current_action == "a"} {
	return [list "add" $current_section]
    }
    
    if {$current_action == "d"} {
	return [list "delete" $current_section]
    }
}

# annotate a file
# this will return a list of lines, where each line is a list of attributes and the content
proc cvs_file_annotate {dir file_name cvsroot} {
    cd $dir
    if {[catch {eval "exec [cvs $cvsroot] annotate $file_name"} errmsg]} {
	set stuff $errmsg
    } else {
	return ""
    }

    set lines [split $stuff "\n"]
    set processed_lines [list]

    foreach line $lines {
	if {![regexp {(\d*\.\d*) *\(([^ ]*) *([^\)]*)\):(.*)} $line all version user date code_line]} {
	    continue
	}

	lappend processed_lines [list [list $version $user $date] $code_line]
    }

    return $processed_lines
}