
#
# This contains defs for doing source code repository online
# We're building an abstraction layer so that we're not
# forever dependent on CVS
#

# The "source code entry" abstraction
# so we can easily tell what is a directory, a file, etc..

proc sdm_sourcecode_make_entry {type path name} {
    return [list $type $path $name]
}

proc sdm_sourcecode_entry_type {entry} {
    return [lindex $entry 0]
}

proc sdm_sourcecode_entry_path {entry} {
    return [lindex $entry 1]
}

proc sdm_sourcecode_entry_name {entry} {
    return [lindex $entry 2]
}

proc sdm_sourcecode_entry_fullpath {entry} {
    return [sdm_sourcecode_entry_path $entry]/[sdm_sourcecode_entry_name $entry]
}

# Displaying an entry as HTML
proc sdm_sourcecode_entry_html {entry} {
    return "([sdm_sourcecode_entry_type $entry]) - [sdm_sourcecode_entry_path $entry]/[sdm_sourcecode_entry_name $entry]"
}

#
# Utility files
#

proc sdm_sourcecode_package_has_repository_p {db package_id} {
    return [database_to_tcl_string $db "select count(*) from package_repositories where package_id=$package_id"]
}

proc set_simple_repository_information {db package_id} {
    uplevel {
	set selection [ns_db 1row $db "select repository_name, file_glob_patterns from package_repositories where package_id=$package_id"]
	set_variables_after_query
    }
}

proc sdm_sourcecode_create_repository {name} {
    ns_mkdir [sdm_source_root]/$name
}

proc sdm_sourcecode_edit_repository {old_name new_name} {
    if {$old_name != $new_name} {
	exec mv [sdm_source_root]/$old_name [sdm_source_root]/$new_name
    }
}

##
## Source code tree abstraction
##

proc sdm_sourcecode_cvs {} {
    return "/usr/local/bin/cvs"
}

proc sdm_source_root {} {
    return "[ad_parameter PathToACS]/data/sdm-source"
}

proc sdm_check_safe {name} {
    if {[string first "../" $name] != -1} {
	# NOT SAFE!
	return 0
    } else {
	return 1
    }
}

proc sdm_sourcecode_list_entries {entry {glob_patterns {}}} {
    if {[string compare [sdm_sourcecode_entry_type $entry] DIRECTORY] != 0} {
	return [list $entry]
    }

    set relpath [sdm_sourcecode_entry_path $entry]/[sdm_sourcecode_entry_name $entry]
    set path [sdm_source_root]/$relpath

    if {![file exists $path]} {
	return ""
    }

    if {![file isdirectory $path]} {
	return ""
    }

    # Get the list of files
    cd $path

    set ls_return [exec ls -l]

    set ls_return_by_line [split $ls_return "\n"]

    set list_of_entries [list]

    set glob_pattern_list [split $glob_patterns " "]

    foreach line $ls_return_by_line {
	if {[regexp {[^ ]+ +[^ ]+.+ +([^ ]+)} $line all filename]} {
	    if {[file isdirectory $path/$filename]} {
		set type DIRECTORY
	    } else {
		set match_flag 0
		foreach glob_pattern $glob_pattern_list {
		    if {[string match $glob_pattern $filename]} {
			set match_flag 1
			break
		    }
		}

		if {!$match_flag} {
		    continue
		}

		set type FILE
	    }

	    lappend list_of_entries [sdm_sourcecode_make_entry $type $relpath $filename]
	}
    }

    return $list_of_entries
    
}

#
#

proc sdm_sourcecode_entry_safe {entry repository} {
    if {![sdm_check_safe [sdm_sourcecode_entry_path $entry]]} {
	return 0
    }

    if {![sdm_check_safe [sdm_sourcecode_entry_name $entry]]} {
	return 0
    }

    set whole_path [sdm_sourcecode_entry_path $entry]/[sdm_sourcecode_entry_name $entry]

    if {![regexp "^/$repository/.*" $whole_path all]} {
	if {![regexp "^/$repository\$" $whole_path all]} {
	    return 0
	}
    }

    return 1
}

proc sdm_sourcecode_entry_has_parent {entry} {
    set path [sdm_sourcecode_entry_path $entry]/[sdm_sourcecode_entry_name $entry]
    
    if {[regexp {^/[^/]+$} $path all]} {
	return 0
    } else {
	return 1
    }
}

proc sdm_sourcecode_entry_parent_entry {entry} {
    if {![sdm_sourcecode_entry_has_parent $entry]} {
	return $entry
    }

    set path [sdm_sourcecode_entry_path $entry]

    if {[regexp {^(.*)/([^/]+)} $path all newpath newname]} {
	return [sdm_sourcecode_make_entry DIRECTORY $newpath $newname]
    } else {
	ns_log Notice "path: $path"
	return ""
    }
}

proc sdm_sourcecode_entry_link {package_id entry} {
    set name [sdm_sourcecode_entry_name $entry]
    if {[string tolower [file extension $name]] == ".html"} {
	return "<a target=other href=sourcecode-entry-contents.tcl?[export_url_vars package_id entry]>view contents</a>"
    } else {
	return ""
    }
}

proc sdm_sourcecode_entry_processed_contents {package_id entry cvsroot} {
    set link [sdm_sourcecode_entry_link $package_id $entry]

    if {![empty_string_p $link]} {
	return $link
    }

    set raw_contents [ns_quotehtml [sdm_sourcecode_entry_get_contents $entry $cvsroot]]

    set colorized_contents [sdm_sourcecode_colorize [sdm_sourcecode_entry_name $entry] $raw_contents]
    set colorized_contents_with_line_numbers [sdm_sourcecode_add_lines_numbers $colorized_contents]

    return "<blockquote><pre>
$colorized_contents_with_line_numbers
</pre>
</blockquote>
"
}

proc sdm_sourcecode_entry_get_contents {entry cvsroot {tag ""}} {
    if {[sdm_sourcecode_entry_type $entry] != "FILE"} {
	return ""
    }

    # set fullpath [sdm_source_root]/[sdm_sourcecode_entry_fullpath $entry]

    # set filestream [open $fullpath "r"]
    # set contents [read $filestream]

    set contents [cvs_checkout_to_string [sdm_sourcecode_entry_fullpath $entry] $cvsroot $tag]

    return $contents
}


proc sdm_sourcecode_add_lines_numbers {contents {count 0}} {
    set newcontents ""
    
    set lines [split $contents "\n"]

    set spaces "     "

    foreach line $lines {
	incr count

	# Do a cute thing to pad the line number
	append newcontents [string range $spaces [string length $count] end]

	append newcontents "<font color=black>$count |</font>  $line\n"
    }

    return $newcontents
}

## More utilities

# Expects a tar'ed and gzipped file.
proc sdm_sourcecode_load_from_file {db package_id file_path} {
    set repository_name [database_to_tcl_string $db "select repository_name from package_repositories where package_id=$package_id"]

    # Remove everything from source directory
    cd [sdm_source_root]
    if {[catch {exec /bin/rm -rf $repository_name} errmsg]} {}
    if {[catch {ns_mkdir [sdm_source_root]/$repository_name} errmsg]} {}

    # Copy the file
    ns_cp $file_path [sdm_source_root]/$repository_name/release.tar.gz

    # CD, Gunzip, Untar
    cd [sdm_source_root]/$repository_name
    if {[catch {exec /usr/local/bin/gunzip release.tar.gz} errmsg]} {}
    if {[catch {exec /usr/local/bin/tar xf release.tar} errmsg]} {}
}


