
##
## The Todo Plugin Architecture
##
## A todo plugin is another module that wants to have its information
## displayed in the form of a bunch of todo items. This plugin architecture
## works by having each submodule register itself by declaring a callback API
## which the todo pages can use to fetch the submodule's information in todo format.
##
## written by Ben Adida (ben@adida.net)

proc todo_plugins_varname {} {
    return "todo_plugins"
}

proc todo_date_prettifier {} {
    return "pretty_relative_date"
}

# Register a todo plugin
proc todo_plugin_register {plugin_name plugin_data} {
    # Simply store this globally
    nsv_set [todo_plugins_varname] $plugin_name $plugin_data
}

# Get the list of plugins
proc todo_get_plugins {} {
    return [nsv_array names [todo_plugins_varname]]
}

# Get a value for a key for a plugin
proc todo_get_plugin_value {plugin_name data_key} {
    # Fetch the plugin ns_set, then the key from the ns_set
    return [ns_set get [nsv_get [todo_plugins_varname] $plugin_name] $data_key]
}


# Get a list of the categories and plugins
# that are available for that user
proc todo_get_personal_plugins_and_categories {db user_id} {
    set return_list [list]

    foreach plugin [todo_get_plugins] {
	# Fetch the categories for that plugin and 
	# that user
	foreach category [[todo_get_plugin_value $plugin GET_PERSONAL_CATEGORIES] $db $user_id] {
	    lappend return_list [list $plugin [lindex $category 0] [lindex $category 1]]
	}
    }

    return $return_list
}

# Get a list of the categories and plugins
# that are available for that user to view about other users.
proc todo_get_other_plugins_and_categories {db user_id} {
    set return_list [list]

    foreach plugin [todo_get_plugins] {
	# Fetch the categories for that plugin and 
	# that user
	foreach category [[todo_get_plugin_value $plugin GET_OTHER_CATEGORIES] $db $user_id] {
	    lappend return_list [list $plugin [lindex $category 0] [lindex $category 1]]
	}
    }

    return $return_list
}

# Get the category name from a category id
proc todo_plugin_get_category_name {db user_id plugin category_id} {
    foreach personal_category [todo_get_personal_plugins_and_categories $db $user_id] {
	if {[lindex $personal_category 0] != $plugin} {
	    continue
	}

	if {[lindex $personal_category 1] == $category_id} {
	    return [lindex $personal_category 2]
	}
    }

    foreach personal_category [todo_get_other_plugins_and_categories $db $user_id] {
	if {[lindex $personal_category 0] != $plugin} {
	    continue
	}

	if {[lindex $personal_category 1] == $category_id} {
	    return [lindex $personal_category 2]
	}
    }
}

proc todo_all_plugins_sql {db user_id date_limit current_url {n_days_to_show_completed_items 0} {order_by ""} {view_completed "both"}} {
    set sql ""

    foreach plugin [todo_get_plugins] {
	set one_plugin_sql [[todo_get_plugin_value $plugin ITEM_LIST_SQL] $db $user_id "" $date_limit [todo_date_prettifier] $n_days_to_show_completed_items $view_completed ""]
	
	if {$sql != ""} {
	    append sql " union "
	}
	
	append sql "($one_plugin_sql)"
    }

    append sql " $order_by"

    return $sql
}
    
# This returns a bunch of table rows, without the start of the table
# or the end of the table.
proc todo_plugin_get_html {db user_id plugin category_id date_limit current_url list_of_headers {item_id_to_edit {}} {view_open_items ""} {n_days_to_show_completed_items 0} {order_by ""} {allow_adding 0}} {

    if {$order_by == ""} {
	set order_by {order by raw_due_date, priority}
    } else {
	# hack to substitue due_date
	# regsub -all due_date $order_by {$due_date} order_by
    }

    if {$view_open_items == ""} {
	set completion_status "both"
    } elseif {$view_open_items == "t"} {
	set completion_status "open"
    } else {
	set completion_status "closed"
    }

    # Get the SQL query from the plugin
    set sql_query [[todo_get_plugin_value $plugin ITEM_LIST_SQL] $db $user_id $category_id "" [todo_date_prettifier] $n_days_to_show_completed_items $completion_status $order_by]

    return [todo_plugin_get_html_by_sql $db $sql_query $current_url $list_of_headers $item_id_to_edit $view_open_items $allow_adding $plugin $category_id]
}

	    
proc todo_plugin_get_html_by_sql {db sql_query current_url list_of_headers {item_id_to_edit {}} {view_open_items ""} {allow_adding 0} {plugin ""} {category_id ""}} {
    # set return_url "[ns_conn url]?[ns_conn query]"
    set return_url $current_url

    if {[string first "?" $return_url] == -1} {
	set return_url "${return_url}?"
    }

    # Perform the query
    ns_log Notice "sql query is $sql_query"
    set selection [ns_db select $db $sql_query]

    # Initialize the HTML
    set return_html ""

    # Loop through
    while {[ns_db getrow $db $selection]} {
	set_variables_after_query
	## We expect the following vars
	## todo_id, item_details, completion_date
	## plugin category_id category

	# Get a whole bunch of data out of the plugin
	set one_item_url [todo_get_plugin_value $plugin ONE_ITEM_URL]
	set one_item_id_key [todo_get_plugin_value $plugin ONE_ITEM_ID_KEY]
	set item_change_allowed_p [todo_get_plugin_value $plugin ITEM_CHANGE_ALLOWED_P]
	set mark_completed_allowed_p [todo_get_plugin_value $plugin MARK_COMPLETED_ALLOWED_P]
	
	if {$item_id_to_edit == $todo_id} {
	    ns_set put $selection n_days [todo_n_days_from_pretty $due_date]
	    append return_html "<FORM METHOD=POST action=plugin-item-edit.tcl>
	    [export_form_vars todo_id return_url plugin]
	    [bt_mergepiece [todo_one_item_form "edit"] $selection]
	    </FORM>\n"
	    continue
	}

	set one_row "<tr><font size=-1>\n"
	
	set actions_html "<td><nobr>"
	# In case it's completed
	if {$completion_date == ""} {
	    set pre_item ""
	    set post_item ""
	    if {$mark_completed_allowed_p} {
		append actions_html "<a href=plugin-item-completed.tcl?[export_url_vars plugin category_id todo_id return_url]><img align=middle height=15 border=0 src=unchecked.gif></a>"
	    }
	    if {$item_change_allowed_p} {
		append actions_html "<a href=${return_url}&item_id_to_edit=$todo_id>edit</a>"
	    }
	} else {
	    set pre_item "<strike>"
	    set post_item "</strike>"
	    if {$mark_completed_allowed_p} {
		append actions_html "<a href=plugin-item-uncompleted.tcl?[export_url_vars plugin category_id todo_id return_url]><img align=middle height=15 border=0 src=checked.gif></a>"
	    }
	    if {$item_change_allowed_p} {
		set return_url $current_url
		append actions_html "<a href=plugin-item-delete.tcl?[export_url_vars return_url plugin category_id todo_id]><img align=middle height=15 border=0 src=delete.gif></a>"
	    }
	}

	append actions_html "</nobr></td>"

	# Go through the headers
	foreach header $list_of_headers {
	    # Special case for list name
	    if {[lindex $header 0] == "list_name"} {
		append one_row "<td>$pre_item<a href=one-plugin-list.tcl?[export_url_vars list_id]>$list_name</a>$post_item</td>"
		continue
	    }

	    # Special case for item details
	    if {[lindex $header 0] == "item_details"} {
		if {[string length $item_details] > 100} {
		    append one_row "<td>$pre_item (<a href=${one_item_url}?$one_item_id_key=$todo_id>details</a>) [string range $item_details 0 100]... (<a target=detail_window href=${one_item_url}?$one_item_id_key=$todo_id>more</a>) $post_item</td>"
		    continue
		} else {
		    append one_row "<td>$pre_item (<a href=${one_item_url}?$one_item_id_key=$todo_id>details</a>) $item_details $post_item</td>"
		    continue
		}
	    }

	    if {[lindex $header 0] == "priority"} {
		append one_row "<td>$pre_item<strong>($priority)</strong>$post_item</td>"
		continue
	    }

	    if {[lindex $header 0] == "category"} {
		append one_row "<td>$pre_item<a href=one-plugin-list.tcl?[export_url_vars category category_id plugin]>$category</a>$post_item</td>"
		continue
	    }
	    
	    append one_row "<td>$pre_item[set [lindex $header 0]]$post_item</td>"
	}
	
	append one_row "$actions_html</font></tr>\n"

	append return_html $one_row
    }

    if {$allow_adding && [todo_get_plugin_value $plugin ITEM_ADD_ALLOWED_P]!="" && [todo_get_plugin_value $plugin ITEM_ADD_ALLOWED_P]} {
	append return_html "<FORM METHOD=POST action=plugin-item-new.tcl>[export_form_vars plugin category_id][todo_one_item_form "add"]</FORM>"
    }

    return $return_html
}    

