--
-- ecommerce-plsql.sql
--
-- by eveander@arsdigita.com, April 1999
--

-- Helper stuff (ben@adida.net)
create function least(numeric,numeric)
returns numeric
as '
DECLARE
	first alias for $1;
	second alias for $2;
BEGIN
	if first < second
	then return first;
	else return second;
	end if;
END;
' language 'plpgsql';

--------------- price calculations -------------------
-------------------------------------------------------

-- just the price of an order, not shipping, tax, or gift certificates
-- this is actually price_charged minus price_refunded
create function ec_total_price (integer) 
returns numeric
AS '
declare
	v_order_id alias for $1;
BEGIN
	RETURN case when sum(price_charged) is null then 0::numeric else sum(price_charged) end - 
	case when sum(price_refunded) is null then 0::numeric else sum(price_refunded) end
		FROM ec_items
		WHERE order_id= v_order_id
		and item_state <> ''void'';
END;
' language 'plpgsql';

-- just the shipping of an order, not price, tax, or gift certificates
-- this is actually total shipping minus total shipping refunded
create function ec_total_shipping(integer)
returns numeric as '
DECLARE
	v_order_id alias for $1;
	order_shipping		numeric;
	item_shipping		numeric;
BEGIN
	select case when shipping_charged is null then 0::numeric else shipping_charged end - 
	case when shipping_refunded is null then 0::numeric else shipping_refunded end
	into order_shipping
	from ec_orders where order_id= v_order_id;

	select case when sum(shipping_charged) is null then 0::numeric else sum(shipping_charged) end - 
	case when sum(shipping_refunded) is null then 0:: numeric else sum(shipping_refunded) end
	into item_shipping
	from ec_items
	where order_id= v_order_id
	and item_state <> ''void'';

	RETURN order_shipping + item_shipping;
END;
' language 'plpgsql';

-- OK
-- just the tax of an order, not price, shipping, or gift certificates
-- this is tax minus tax refunded
create function ec_total_tax(integer)
returns numeric as '
DECLARE
	v_order_id alias for $1;
	order_tax		numeric;
	item_price_tax		numeric;
	item_shipping_tax	numeric;
BEGIN
	select case when shipping_tax_charged is null then 0::numeric else shipping_tax_charged end - 
	case when shipping_tax_refunded is null then 0::numeric else shipping_tax_refunded end
	into order_tax
	from ec_orders
	where order_id= v_order_id;

	select case when sum(price_tax_charged) is null then 0::numeric else sum(price_tax_charged) end -
	case when sum(price_tax_refunded) is null then 0::numeric else sum(price_tax_refunded) end
	into item_price_tax
	from ec_items
	where order_id= v_order_id
	and item_state <> ''void'';

	select case when sum(shipping_tax_charged) is null then 0::numeric else sum(shipping_tax_charged) end - 
	case when sum(shipping_tax_refunded) is null then 0::numeric else sum(shipping_tax_refunded) end
	into item_shipping_tax
	from ec_items
	where order_id= v_order_id;

	RETURN order_tax + item_price_tax + item_shipping_tax;
END;
' language 'plpgsql';

-- OK
-- just the price of a shipment, not shipping, tax, or gift certificates
-- this is the price charged minus the price refunded of the shipment
create function ec_shipment_price(integer)
returns numeric
as '
DECLARE
	v_shipment_id alias for $1;
	shipment_price	    numeric;
BEGIN
	SELECT case when sum(price_charged) is null then 0::numeric else sum(price_charged) end - 
	case when sum(price_refunded) is null then 0::numeric else sum(price_refunded) end 
	into shipment_price
	FROM ec_items
	WHERE shipment_id= v_shipment_id
	and item_state <> ''void'';

	RETURN shipment_price;
END;
' language 'plpgsql';

-- OK
-- just the shipping charges of a shipment, not price, tax, or gift certificates
-- note: the base shipping charge is always applied to the first shipment in an order.
-- this is the shipping charged minus the shipping refunded
create function ec_shipment_shipping(integer)
returns numeric
as '
DECLARE
	v_shipment_id alias for $1;
	item_shipping		numeric;
	base_shipping		numeric;
	v_order_id		ec_orders.order_id%TYPE;
	min_shipment_id		ec_shipments.shipment_id%TYPE;
BEGIN
	SELECT order_id into v_order_id FROM ec_shipments where shipment_id= v_shipment_id;
	SELECT min(shipment_id) into min_shipment_id FROM ec_shipments where order_id= v_order_id;
	IF v_shipment_id= min_shipment_id THEN
		SELECT case when shipping_charged is null then 0::numeric else shipping_charged end - 
		case when shipping_refunded is null then 0::numeric else shipping_refunded end 
		into base_shipping FROM ec_orders where order_id= v_order_id;
	ELSE
		base_shipping := 0;
	END IF;
	SELECT  case when sum(shipping_charged) is null then 0::numeric else sum(shipping_charged) end - 
	case when sum(shipping_refunded) is null then 0::numeric else sum(shipping_refunded) end 
	into item_shipping FROM ec_items where shipment_id= v_shipment_id and item_state <> ''void'';
	RETURN item_shipping + base_shipping;
END;
' language 'plpgsql';

-- OK
-- just the tax of a shipment, not price, shipping, or gift certificates
-- note: the base shipping tax charge is always applied to the first shipment in an order.
-- this is the tax charged minus the tax refunded
create function ec_shipment_tax(integer)
returns numeric
as '
DECLARE
	v_shipment_id alias for $1;
	item_price_tax		numeric;
	item_shipping_tax	numeric;
	base_shipping_tax	numeric;
	v_order_id		ec_orders.order_id%TYPE;
	min_shipment_id		ec_shipments.shipment_id%TYPE;
BEGIN
	SELECT order_id into v_order_id FROM ec_shipments where shipment_id= v_shipment_id;
	SELECT min(shipment_id) into min_shipment_id FROM ec_shipments where order_id= v_order_id;
	IF v_shipment_id= min_shipment_id THEN
		SELECT coalesce(shipping_tax_charged,0::numeric) - coalesce(shipping_tax_refunded,0::numeric) into base_shipping_tax FROM ec_orders where order_id= v_order_id;
	ELSE
		base_shipping_tax := 0;
	END IF;
	SELECT coalesce(sum(price_tax_charged),0::numeric) - coalesce(sum(price_tax_refunded),0::numeric) into item_price_tax FROM ec_items where shipment_id= v_shipment_id and item_state <> ''void'';
	SELECT coalesce(sum(shipping_tax_charged),0::numeric) - coalesce(sum(shipping_tax_refunded),0::numeric) into item_shipping_tax FROM ec_items where shipment_id= v_shipment_id and item_state <> ''void'';
	RETURN item_price_tax + item_shipping_tax + base_shipping_tax;
END;
' language 'plpgsql';

-- OK
-- the gift certificate amount used on one order
create function ec_order_gift_cert_amount(integer)
returns numeric
as '
DECLARE
	v_order_id alias for $1;
	gift_cert_amount	numeric;
BEGIN
	select case when sum(amount_used) is null then 0::numeric else sum(amount_used) end - 
	case when sum(amount_reinstated) is null then 0::numeric else sum(amount_reinstated) end
	into gift_cert_amount
	from ec_gift_certificate_usage
	where order_id= v_order_id;

	return gift_cert_amount;
END;
' language 'plpgsql';


-- OK
-- tells how much of the gift certificate amount used on the order is to be applied
-- to a shipment (it's applied chronologically)
create function ec_shipment_gift_certificate (integer)
returns numeric
as '
DECLARE
	v_shipment_id alias for $1;
	v_order_id		ec_orders.order_id%TYPE;
	gift_cert_amount	numeric;
	past_ship_amount	numeric;
BEGIN
	SELECT order_id into v_order_id FROM ec_shipments WHERE shipment_id= v_shipment_id;
	gift_cert_amount := ec_order_gift_cert_amount(v_order_id);

	SELECT coalesce(sum(ec_shipment_price(shipment_id)) + sum(ec_shipment_shipping(shipment_id))+sum(ec_shipment_tax(shipment_id)),0::numeric) into past_ship_amount FROM ec_shipments WHERE order_id = v_order_id and shipment_id <> v_shipment_id;

	IF past_ship_amount > gift_cert_amount THEN
		return 0;
	ELSE
		return least(gift_cert_amount - past_ship_amount, coalesce(ec_shipment_price(v_shipment_id) + ec_shipment_shipping(v_shipment_id) + ec_shipment_tax(v_shipment_id),0::numeric));
	END IF;
END;
' language 'plpgsql';

-- OK
-- this can be used for either an item or order
-- given price and shipping, computes tax that needs to be charged (or refunded)
-- order_id is an argument so that we can get the usps_abbrev (and thus the tax rate),
--
-- OUTER JOIN HACK by BMA: I just took out the outer join for now, it doesn't look very necessary, we'll see.

create function ec_tax (numeric, numeric, integer)
returns numeric
as '
DECLARE
	v_price alias for $1;
	v_shipping alias for $2;
	v_order_id alias for $3;
	taxes			ec_sales_tax_by_state%ROWTYPE;
BEGIN
	SELECT t.* into taxes
	FROM ec_orders o, ec_addresses a, ec_sales_tax_by_state t
	WHERE o.shipping_address=a.address_id
	AND a.usps_abbrev=t.usps_abbrev
	AND o.order_id= v_order_id;
	
	IF coalesce(taxes.shipping_p,''f'') = ''f'' THEN
		return coalesce(taxes.tax_rate,0)::numeric * v_price;
	ELSE
		return coalesce(taxes.tax_rate,0)::numeric * (v_price + v_shipping);
	END IF;
END;
' language 'plpgsql';

-- OK
-- total order cost (price + shipping + tax - gift certificate)
-- this should be equal to the amount that the order was authorized for
-- (if no refunds have been made)
create function ec_order_cost(integer) 
returns numeric
as '
DECLARE
	v_order_id alias for $1;
	v_price		numeric;
	v_shipping	numeric;
	v_tax		numeric;
	v_certificate	numeric;
BEGIN
	v_price := ec_total_price(v_order_id);
	v_shipping := ec_total_shipping(v_order_id);
	v_tax := ec_total_tax(v_order_id);
	v_certificate := ec_order_gift_cert_amount(v_order_id);

	return v_price + v_shipping + v_tax - v_certificate;
END;
' language 'plpgsql';

-- OK
-- total shipment cost (price + shipping + tax - gift certificate)
create function ec_shipment_cost(integer)
returns numeric
as '
DECLARE
	v_shipment_id alias for $1;
	v_price		numeric;
	v_shipping	numeric;
	v_certificate	numeric;
	v_tax		numeric;
BEGIN
	v_price := ec_shipment_price(v_shipment_id);
	v_shipping := ec_shipment_shipping(v_shipment_id);
	v_tax := ec_shipment_tax(v_shipment_id);
	v_certificate := ec_shipment_gift_certificate(v_shipment_id);

	return v_price + v_shipping - v_certificate + v_tax;
END;
' language 'plpgsql';

-- OK
-- total amount refunded on an order so far
create function ec_total_refund(integer)
returns numeric
as '
DECLARE
	v_order_id alias for $1;
	v_order_refund	numeric;
	v_items_refund	numeric;
BEGIN
	select coalesce(shipping_refunded,0) + coalesce(shipping_tax_refunded,0) into v_order_refund from ec_orders where order_id= v_order_id;
	select sum(coalesce(price_refunded,0)) + sum(coalesce(shipping_refunded,0)) + sum(coalesce(price_tax_refunded,0)) + sum(coalesce(shipping_tax_refunded,0)) into v_items_refund from ec_items where order_id= v_order_id;
	return v_order_refund + v_items_refund;
END;
' language 'plpgsql';

-------------- end price calculations -----------------
-------------------------------------------------------


----------- gift certificate procedures ---------------
-------------------------------------------------------
  
--
-- BMA (PGsql port)
-- Postgres is way cooler than Oracle with MVCC, which allows it
-- to have triggers updating the same table. Thus, we get rid of this
-- trio crap and we have a simple trigger for everything.

create function trig_ec_cert_amount_remains() 
returns opaque
as '
DECLARE
	bal_amount_used		numeric;
	original_amount		numeric;
BEGIN
	select amount into original_amount
	from ec_gift_certificates where gift_certificate_id= NEW.certificate_id for update;

	select coalesce(sum(amount_used), 0) - coalesce(sum(amount_reinstated), 0)
	into bal_amount_used
	from ec_gift_certificate_usage
	where gift_certificate_id= NEW.gift_certificate_id;

	UPDATE ec_gift_certificates 
	SET amount_remaining_p = case when amount > bal_amount_used then ''t'' else ''f'' end
	WHERE gift_certificate_id = gift_certificate_rec.gift_certificate_id;
END;
' language 'plpgsql';

create trigger ec_cert_amount_remains
before update on ec_gift_certificate_usage
for each row
execute procedure trig_ec_cert_amount_remains();		


-- OK
-- calculates how much a user has in their gift certificate account
create function ec_gift_certificate_balance(integer)
returns numeric
as '
DECLARE
	v_user_id alias for $1;
	original_amount			numeric;
	total_amount_used		numeric;
BEGIN
	SELECT case when sum(amount) is null then 0::numeric else sum(amount) end
	into original_amount
	FROM ec_gift_certificates_approved
	WHERE user_id= v_user_id
	AND amount_remaining_p=''t''
	AND expires > sysdate();
	
	SELECT case when sum(u.amount_used) is null then 0::numeric else sum(u.amount_used) end - 
	case when sum(u.amount_reinstated) is null then 0::numeric else sum(u.amount_reinstated) end
	into total_amount_used
	FROM ec_gift_certificates_approved c, ec_gift_certificate_usage u
	WHERE c.gift_certificate_id=u.gift_certificate_id
	AND c.user_id= v_user_id
	AND c.amount_remaining_p=''t''
	AND c.expires> sysdate();

	RETURN original_amount - total_amount_used;
END;
' language 'plpgsql';

-- OK
-- Returns price + shipping + tax - gift certificate amount applied
-- for one order.
-- Requirement: ec_orders.shipping_charged, ec_orders.shipping_tax_charged,
-- ec_items.price_charged, ec_items.shipping_charged, ec_items.price_tax_chaged,
-- and ec_items.shipping_tax_charged should already be filled in.

create function ec_order_amount_owed(integer)
returns numeric
as '
DECLARE
	v_order_id alias for $1;
	pre_gc_amount_owed		numeric;
	gc_amount			numeric;
BEGIN
	pre_gc_amount_owed := ec_total_price(v_order_id) + ec_total_shipping(v_order_id) + ec_total_tax(v_order_id);
	gc_amount := ec_order_gift_cert_amount(v_order_id);

	RETURN pre_gc_amount_owed - gc_amount;
END;
' language 'plpgsql';

-- OK
-- the amount remaining in an individual gift certificate
create function gift_certificate_amount_left (integer)
returns numeric
as '
DECLARE
	v_gift_certificate_id alias for $1;
	original_amount		numeric;
	amount_used		numeric;
BEGIN
	SELECT coalesce(sum(amount_used),0) - coalesce(sum(amount_reinstated),0)
	into amount_used
	FROM ec_gift_certificate_usage
	WHERE gift_certificate_id = v_gift_certificate_id;
	
	SELECT amount
	into original_amount
	FROM ec_gift_certificates
	WHERE gift_certificate_id = v_gift_certificate_id;

	RETURN original_amount - amount_used;
END;
' language 'plpgsql';

-- I DON'T USE THIS PROCEDURE ANYMORE BECAUSE THERE'S A MORE
-- FAULT-TOLERANT TCL VERSION
-- This applies gift certificate balance to an entire order
-- by iteratively applying unused/unexpired gift certificates
-- to the order until the order is completely paid for or
-- the gift certificates run out.
-- Requirement: ec_orders.shipping_charged, ec_orders.shipping_tax_charged,
-- ec_items.price_charged, ec_items.shipping_charged, ec_items.price_tax_charged,
-- ec_items.shipping_tax_charged should already be filled in.
-- Call this within a transaction.
-- create or replace procedure ec_apply_gift_cert_balance (v_order_id IN integer, v_user_id IN integer)
-- IS
-- 	CURSOR gift_certificate_to_use_cursor IS
-- 		SELECT *
-- 		FROM ec_gift_certificates_approved
-- 		WHERE user_id = v_user_id
-- 		AND (expires is null or sysdate - expires < 0)
-- 		AND amount_remaining_p = 't'
--                 ORDER BY expires;
-- 	amount_owed			number;
-- 	gift_certificate_balance	number;
-- 	certificate			ec_gift_certificates_approved%ROWTYPE;
-- BEGIN
-- 	gift_certificate_balance := ec_gift_certificate_balance(v_user_id);
-- 	amount_owed := ec_order_amount_owed(v_order_id);

--         OPEN gift_certificate_to_use_cursor;
-- 	WHILE amount_owed > 0 and gift_certificate_balance > 0
-- 		LOOP
-- 			FETCH gift_certificate_to_use_cursor INTO certificate;

-- 	        	INSERT into ec_gift_certificate_usage
-- 			(gift_certificate_id, order_id, amount_used, used_date)
-- 			VALUES
-- 			(certificate.gift_certificate_id, v_order_id, least(gift_certificate_amount_left(certificate.gift_certificate_id), amount_owed), sysdate);

-- 			gift_certificate_balance := ec_gift_certificate_balance(v_user_id);
-- 			amount_owed := ec_order_amount_owed(v_order_id);	
-- 		END LOOP;
--         CLOSE gift_certificate_to_use_cursor;
-- END ec_apply_gift_cert_balance;
-- /
-- show errors


-- OK
-- reinstates all gift certificates used on an order (as opposed to
-- individual items), e.g. if the order was voided or an auth failed

create function ec_reinst_gift_cert_on_order(integer)
returns integer
as '
DECLARE
	v_order_id alias for $1;
BEGIN
	insert into ec_gift_certificate_usage
	(gift_certificate_id, order_id, amount_reinstated, reinstated_date)
	select gift_certificate_id, v_order_id, coalesce(sum(amount_used),0)-coalesce(sum(amount_reinstated),0), sysdate()
	from ec_gift_certificate_usage
	where order_id= v_order_id
	group by gift_certificate_id;

	return 0;
END;
' language 'plpgsql';
	
-- Given an amount to refund to an order, this tells
-- you how much of that is to be refunded in cash (as opposed to 
-- reinstated in gift certificates).  Then you know you have to
-- go and reinstate v_amount minus (what this function returns)
-- in gift certificates.
-- (when I say cash I'm really talking about credit card
-- payment -- as opposed to gift certificates)

-- Call this before inserting the amounts that are being refunded
-- into the database.
create function ec_cash_amount_to_refund (numeric, integer)
returns numeric
as '
DECLARE
	v_amount alias for $1;
	v_order_id alias for $2;
	amount_paid			numeric;
	items_amount_paid		numeric;
	order_amount_paid		numeric;
	amount_refunded			numeric;
	curr_gc_amount			numeric;
	max_cash_refundable		numeric;
	cash_to_refund			numeric;
BEGIN
	-- the maximum amount of cash refundable is equal to
	-- the amount paid (in cash + certificates) for shipped items only (since
	--  money is not paid until an item actually ships)
	-- minus the amount refunded (in cash + certificates) (only occurs for shipped items)
	-- minus the current gift certificate amount applied to this order
	-- or 0 if the result is negative

	select sum(coalesce(price_charged,0)) + sum(coalesce(shipping_charged,0)) + sum(coalesce(price_tax_charged,0)) + sum(coalesce(shipping_tax_charged,0)) into items_amount_paid from ec_items where order_id= v_order_id and shipment_id is not null and item_state <> ''void'';

	select coalesce(shipping_charged,0) + coalesce(shipping_tax_charged,0) into order_amount_paid from ec_orders where order_id= v_order_id;

	amount_paid := items_amount_paid + order_amount_paid;
	amount_refunded := ec_total_refund(v_order_id);
	curr_gc_amount := ec_order_gift_cert_amount(v_order_id);
	
	max_cash_refundable := amount_paid - amount_refunded - curr_gc_amount;
	cash_to_refund := least(max_cash_refundable, v_amount);

	RETURN cash_to_refund;
END;
' language 'plpgsql';

-- The amount of a given gift certificate used on a given order.
-- This is a helper function for ec_gift_cert_unshipped_amount.
create function ec_one_gift_cert_on_one_order (integer,integer)
returns numeric
as '
DECLARE
	v_gift_certificate_id alias for $1;
	v_order_id alias for $2;
	bal_amount_used		numeric;
BEGIN
	select coalesce(sum(amount_used),0)-coalesce(sum(amount_reinstated),0) into bal_amount_used
	from ec_gift_certificate_usage
	where order_id= v_order_id
	and gift_certificate_id= v_gift_certificate_id;

	RETURN bal_amount_used;

END;
' language 'plpgsql';

-- The amount of all gift certificates used on a given order that
-- expire before* a given gift certificate (*in the event that two
-- expire at precisely the same time, the one with a higher
-- gift_certificate_id is defined to expire last).
-- This is a helper function for ec_gift_cert_unshipped_amount.
create function ec_earlier_certs_on_one_order (integer, integer)
returns numeric 
as '
DECLARE
	v_gift_certificate_id alias for $1;
	v_order_id alias for $2;
	bal_amount_used		numeric;
BEGIN
	select coalesce(sum(u.amount_used),0)-coalesce(sum(u.amount_reinstated),0) into bal_amount_used
	from ec_gift_certificate_usage u, ec_gift_certificates g, ec_gift_certificates g2
	where u.gift_certificate_id=g.gift_certificate_id
	and g2.gift_certificate_id= v_gift_certificate_id
	and u.order_id= v_order_id
	and (g.expires < g2.expires or (g.expires = g2.expires and g.gift_certificate_id < g2.gift_certificate_id));

	return bal_amount_used;
END;
' language 'plpgsql';

-- The amount of a gift certificate that is applied to the upshipped portion of an order.
-- This is a helper function for ec_gift_cert_unshipped_amount.
create function ec_cert_unshipped_one_order (integer, integer)
returns numeric 
as ' 
DECLARE
	v_gift_certificate_id alias for $1;
	v_order_id alias for $2;
	total_shipment_cost	numeric;
	earlier_certs		numeric;
	total_tied_amount	numeric;
BEGIN
	select coalesce(sum(coalesce(ec_shipment_price(shipment_id),0) + coalesce(ec_shipment_shipping(shipment_id),0) + coalesce(ec_shipment_tax(shipment_id),0)),0) into total_shipment_cost
	from ec_shipments
	where order_id= v_order_id;

	earlier_certs := ec_earlier_certs_on_one_order(v_gift_certificate_id, v_order_id);

	IF total_shipment_cost <= earlier_certs THEN
		total_tied_amount := ec_one_gift_cert_on_one_order(v_gift_certificate_id, v_order_id);
	ELSIF total_shipment_cost > earlier_certs + ec_one_gift_cert_on_one_order(v_gift_certificate_id, v_order_id) THEN
		total_tied_amount := 0;
	ELSE
		total_tied_amount := ec_one_gift_cert_on_one_order(v_gift_certificate_id, v_order_id) - (total_shipment_cost - earlier_certs);
	END IF;

	RETURN total_tied_amount;		
END;
' language 'plpgsql';

-- Returns the amount of a gift certificate that is applied to the unshipped portions of orders
-- (this amount is still considered "outstanding" since revenue, and thus gift certificate usage,
-- isn't recognized until the items ship).
create function ec_gift_cert_unshipped_amount (integer)
returns numeric
as '
DECLARE
	v_gift_certificate_id alias for $1;
	tied_but_unshipped_amount	numeric;
BEGIN
	select coalesce(sum(ec_cert_unshipped_one_order(v_gift_certificate_id,order_id)),0) into tied_but_unshipped_amount
	from ec_orders
	where order_id in (select unique order_id from ec_gift_certificate_usage where gift_certificate_id= v_gift_certificate_id);

	return tied_but_unshipped_amount;
END;
' language 'plpgsql';

---------- end gift certificate procedures ------------
-------------------------------------------------------

-- Thing to help out with outer joins

create function ec_items_reportable_count(integer)
returns integer
as '
DECLARE
	v_product_id alias for $1;
BEGIN
	return count(*) from ec_items_reportable where product_id= v_product_id;
END;
' language 'plpgsql';


create function ec_comments_count(integer)
returns integer
as '
DECLARE
	v_product_id alias for $1;
BEGIN
	return count(*) from ec_product_comments where product_id= v_product_id;
END;
' language 'plpgsql';


create function ec_shipment_status(integer, integer)
returns varchar
as '
DECLARE
	v_order_id alias for $1;
	v_shipment_id alias for $2;
	n_items_ordered integer;
	n_items_shipped integer;
BEGIN
	select coalesce(count(*),0) into n_items_ordered from ec_items where order_id= v_order_id;
	select coalesce(count(*),0) into n_items_shipped from ec_items where shipment_id= v_shipment_id;

	if n_items_ordered = n_items_shipped
	then return ''Full'';
	else return ''Partial'';
	end if;
END;
' language 'plpgsql';


create function ec_shipment_n_items(integer)
returns varchar
as '
DECLARE
	v_shipment_id alias for $1;
BEGIN
	return coalesce(count(*),0) from ec_items where shipment_id= v_shipment_id;
END;
' language 'plpgsql';


create function ec_category_product_count(integer)
returns integer
as '
DECLARE
	v_category_id alias for $1;
BEGIN
	return count(*) from ec_category_product_map where category_id= v_category_id;
END;
' language 'plpgsql';

create function cybercash_to_date(varchar)
returns datetime
as '
DECLARE
	v_str alias for $1;
	new_str varchar(20);
BEGIN
	if v_str=''''
	then return null;
	end if;

	new_str:= substr(v_str,1,4) || ''-'' || substr(v_str,5,2) || ''-'' || substr(v_str,7,2) ||
		  '' '' || substr(v_str,9,2) || '':'' || substr(v_str,11,2) || '':'' ||
		  substr(v_str,12,2);
	
	return new_str::datetime;
END;
' language 'plpgsql';
