
--
-- Integration of todo/ticket
--

create table todo_item_ticket_map (
       item_id			  integer not null references todo_items,
       issue_id			  integer not null references ticket_issues,
       primary key(item_id, issue_id)
);


alter table todo_items add (
      no_trigger       integer
);

alter table ticket_issues add (
      no_trigger	  integer
);


-- a procedure to get the todo_list_id for issues for a particular user
create function todo_get_ticket_list_id(integer)
returns integer as '
DECLARE
	v_user_id alias for $1;
	v_check integer;
	v_list_id integer;
BEGIN
	select count(*) into v_check
	from todo_lists where user_id=v_user_id and list_name=''TICKETS'';

	if v_check=0
	THEN 
	     select nextval(''todo_list_id_sequence'') into v_list_id from dual;
	     insert into todo_lists (list_id, user_id, list_name, creation_date)
	     VALUES (v_list_id, v_user_id, ''TICKETS'', sysdate);
	ELSE
		select list_id into v_list_id from todo_lists where user_id=v_user_id
		and list_name=''TICKETS'';
	END IF;

	return(v_list_id);
END;
' language 'plpgsql';


-- a procedure to turn an issue into a todo
create function issue_to_todo(integer, integer, datetime, datetime, varchar)
returns void as '
DECLARE
	v_issue_id alias for $1;
	v_user_id alias for $2;
	deadline alias for $3;
	close alias for $4;
	subject alais for $5;
	v_item_id integer;
	v_list_id integer;
	v_check integer;
BEGIN
	v_list_id:= todo_get_ticket_list_id(v_user_id);

	select count(*) into v_check 
	from todo_items, todo_item_ticket_map
	where list_id= v_list_id and issue_id=v_issue_id 
	and todo_items.item_id= todo_item_ticket_map.item_id;

	if v_check = 0
	THEN 
	     select nextval(''todo_item_id_sequence'') into v_item_id from dual;
	     INSERT INTO todo_items (item_id, list_id, priority, creation_date,
				    due_date, completion_date, item_details) VALUES
				    (v_item_id, v_list_id, 1, sysdate(),
				    deadline, close,
				    ''<a href=/ticket/issue-view.tcl?msg_id='' || v_issue_id || ''>TICKET '' || v_issue_id || ''</a>: '' || subject);
	     INSERT INTO todo_item_ticket_map (item_id, issue_id) VALUES (v_item_id, v_issue_id);
	ELSE
		select todo_items.item_id into v_item_id
		from todo_items, todo_item_ticket_map
		where list_id= v_list_id and issue_id=v_issue_id 
		and todo_items.item_id= todo_item_ticket_map.item_id;

	     UPDATE todo_items set
	     due_date= deadline,
	     completion_date= close,
	     no_trigger= 1
	     WHERE item_id=v_item_id;
	END IF;
END;
' language 'plpgsql';


create function issue_remove_todo (integer, integer)
returns void as '
DECLARE
	v_issue_id alias for $1;
	v_user_id alias for $2;
	v_item_id INTEGER;
	v_list_id INTEGER;
	v_check INTEGER;
BEGIN
	v_list_id:= todo_get_ticket_list_id(v_user_id);

	select count(*) into v_check 
	from todo_items, todo_item_ticket_map
	where list_id= v_list_id and issue_id=v_issue_id 
	and todo_items.item_id= todo_item_ticket_map.item_id;

	if v_check > 0
	then
		select todo_items.item_id into v_item_id
		from todo_items, todo_item_ticket_map
		where list_id= v_list_id and issue_id=v_issue_id 
		and todo_items.item_id= todo_item_ticket_map.item_id;

		delete from todo_item_ticket_map where issue_id=v_issue_id
		and item_id= v_item_id;
		delete from todo_items where item_id=v_item_id;
	END IF;
END;
' language 'plpgsql';

-- a trigger when a user is assigned
create function trig_ticket_update_todo() returns opaque
as '
DECLARE
	CURSOR loop_through_users FOR
	select user_id from ticket_issue_assignments where msg_id=OLD.msg_id;
BEGIN
	IF NEW.no_trigger is not null
	THEN
		NEW.no_trigger:= null;
		RETURN;
	END IF;

	FOR user_data IN loop_through_users LOOP
	    perform issue_to_todo(OLD.msg_id, user_data.user_id, NEW.deadline, NEW.fix_date, NEW.one_line);
	END LOOP;

	RETURN NEW;
END;
' language 'plpgsql';

create trigger ticket_update_todo
BEFORE UPDATE on ticket_issues
FOR EACH ROW
execute procedure trig_ticket_update_todo();


-- trigger when removing an issue
create function trig_ticket_delete_todo() returns opaque
as ' 
DECLARE
	CURSOR loop_through_users FOR
	select user_id from ticket_issue_assignments where msg_id=OLD.msg_id;
BEGIN
	FOR user_data IN loop_through_users LOOP
	    issue_remove_todo(OLD.msg_id, user_data.user_id);
	END LOOP;
	
	RETURN NEW;
END;
' language 'plpgsql';

create trigger ticket_delete_todo
BEFORE DELETE on ticket_issues
FOR EACH ROW
EXECUTE procedure trig_ticket_delete_todo();

-- trigger on assignments
create function trig_insert_ticket_assignment() returns opaque
as '
DECLARE
	v_ticket_issue ticket_issues%ROWTYPE;
BEGIN
	select * into v_ticket_issue 
	from ticket_issues
	where msg_id= NEW.msg_id;

	perform issue_to_todo(NEW.msg_id, NEW.user_id, v_ticket_issue.deadline, v_ticket_issue.close_date, v_ticket_issue.one_line);
	
	return NEW;
END;
' language 'plpgsql';

CREATE trigger insert_ticket_assignment
AFTER INSERT on ticket_issue_assignments
FOR EACH ROW
execute procedure trig_insert_ticket_assignment();

-- delete an assignment
create function trig_delete_ticket_assignment() returns opaque
as '
DECLARE
BEGIN
	perform issue_remove_todo(OLD.msg_id, OLD.user_id);
	return NEW;
END;
' language 'plpgsql';

CREATE trigger delete_ticket_assignment
BEFORE DELETE on ticket_issue_assignments
FOR EACH ROW
execute procedure trig_delete_ticket_assignment();

-- trigger on todo update
create function trig_todo_update_ticket() returns opaque
as '
DECLARE
	v_check INTEGER;
	v_issue_id INTEGER;
	v_user_id  INTEGER;
BEGIN
	IF NEW.no_trigger is not null
	THEN
		NEW.no_trigger:= null;
		RETURN NULL;
	END IF;

	select count(*) into v_check from todo_item_ticket_map
	where item_id=OLD.item_id;

	if v_check > 0
	THEN
		select user_id into v_user_id from todo_lists where list_id=NEW.list_id;

		select issue_id into v_issue_id from todo_item_ticket_map
		where item_id=OLD.item_id;

		update ticket_issues set
		deadline= NEW.due_date,
		status= case when NEW.completion_date is NULL then ''open'' else ''fixed waiting approval'' end,
		modification_time= sysdate(),
		last_modified_by= v_user_id,
		no_trigger= 1
		where
		msg_id= v_issue_id;
	END IF;
	
	RETURN NEW;
END;
' language 'plpgsql';

CREATE trigger todo_update_ticket
BEFORE UPDATE on todo_items
FOR EACH ROW
execute procedure trig_todo_update_ticket();

-- a trigger for when a todo is deleted from the todo-list
create function trig_todo_delete_ticket() returns opaque 
as '
DECLARE
BEGIN
	delete from todo_item_ticket_map where item_id= OLD.item_id;
END;
' language 'plpgsql';


create trigger todo_delete_ticket
BEFORE DELETE ON todo_items
FOR EACH ROW
execute procedure trig_todo_delete_ticket();