# edit-page-permissions.tcl,v 1.4.2.1 2000/02/03 09:53:36 ron Exp
# requires: on_what_id, on_which_table OR permissions_id
# optional: page_url, page_name, return_url, return_name

# page_url is the URL of the page whose permissions we are editing
# page_name is the name of the page whose permissions we are
#    editing (used for prettier UI)
# return_url is the URL of the page calling the edit permissions page
# return_name is the name of the page calling the edit permissions page

# if return_url/return_name are empty, we assume page_url/page_name

ad_page_variables {
    {on_what_id {}}
    {on_which_table {}}
    {permissions_id {}}
    {page_url {}}
    {page_name {}}
    {return_url {}}
    {return_name {}}
    {orderby {}}
}

validate_integer on_what_id $on_what_id
validate_integer permissions_id $permissions_id

set user_id [ad_verify_and_get_user_id]
set group_id [ad_get_group_id]

set db [ns_db gethandle]

set error_count 0
set error_msg ""

if { [info exists permissions_id] && ![empty_string_p $permissions_id] } {
    set not_know_id_p 0
} else {
    set not_know_id_p 1
}

if { $not_know_id_p } {
    if { ![info exists on_what_id] || [empty_string_p $on_what_id] } {
	incr error_count
	append error_msg "<li>We need to know which particular row of the table you want to access."
    }
    
    if { ![info exists on_which_table] || [empty_string_p $on_which_table] } {
	incr error_count
	append error_msg "<li>We need to know which table the page accesses."
    }

    if { $error_count == 0 } {
	set permissions_id [ad_g_permissions_id $db $on_what_id $on_which_table]
    }
    if { ![info exists permissions_id] || $permissions_id == "" } {
	incr error_count
	append error_msg "<li>The permissions record was not found."
    }
}

if { $error_count > 0 } {
    ad_return_complaint $error_count $error_msg
    return
}

if { ![ad_g_owner_p $db $permissions_id $user_id $group_id] } {
    ad_return_error "Unauthorized" "You are not authorized to edit the permissions for this page."
    return
}

# create the page URL/document name information
if { ![info exists page_url] || [empty_string_p $page_url] } {
    set page_name "this page"
    set page_url_info "this page"
} elseif { [info exists page_name] && ![empty_string_p $page_name] } {
    set page_name $page_name
    set page_url_info "<a href=\"$page_url\">$page_name</a>"
} else {
    set page_url_info "<a href=\"$page_url\">$page_url</a>"
}

# if we don't have all the info we need for return_name
if { [info exists return_name] && ![empty_string_p $return_name] && [info exists return_url] && ![empty_string_p $return_url] } {
    set navbar "[ad_context_bar_ws_or_index [list "$return_url" "$return_name"] "Edit Permissions"]"
} else {
    set navbar "[ad_context_bar_ws_or_index [list "$page_url" "$page_name"] "Edit Permissions"]"
}

# set up data for dimensional slider
set dimensional_list {
    { dim "" all {
	{users "Users" {where "pum.group_id is null"}}
	{groups "Groups" {where "pum.user_id is null"}}
	{owners "Owners" {where "owner_p = 't'"}}
	{all "All" {}}
}   }
}

# set up the table definition
set export_url_vars [export_url_vars permissions_id page_url page_name return_url return_name]

# some helper procedures that will be called by ad_table
proc get_short_name g {
    if { ![empty_string_p $g] } {
	set db [ns_db gethandle subquery] 
	set return_value [database_to_tcl_string $db "select short_name from user_groups where group_id = $g"]
	ns_db releasehandle $db
    } else {
	set return_value ""
    }
    return $return_value
}

proc give_UG_name_url_admin {user_id group_id first_names last_name group_name} {
    if { [empty_string_p $group_name] } {
	return "<td><a href=\"/admin/users/one.tcl?user_id=$user_id\">$first_names $last_name</a></td>"
    } else {
	return "<td><a href=\"/admin/ug/group.tcl?group_id=$group_id\">$group_name</a></td>"
    }
}

proc give_UG_name_url {user_id group_id first_names last_name group_name} {
    if { [empty_string_p $group_name] } {
	return "<td><a href=\"/shared/community-member.tcl?user_id=$user_id\">$first_names $last_name</a></td>"
    } else {
	return "<td><a href=\"/[ad_parameter GroupsDirectory ug]/[ad_urlencode [get_short_name $group_id]]/\">$group_name</a></td>"
    }
}

if { [ad_permission_p $db "" "" administrator $user_id $group_id] } {
    set table_def {
	{fullname "User/Group Name" {upper(last_name) $order, upper(first_names) $order, upper(group_name)} "[give_UG_name_url_admin $user_id $group_id $first_names $last_name $group_name]"}
    }
} else {
    set table_def {
	{fullname "User/Group Name" {upper(last_name) $order, upper(first_names) $order, upper(group_name)} "[give_UG_name_url $user_id $group_id $first_names $last_name $group_name]"}
    }
}

    
if { [ad_parameter GroupPermissionsP gp] } {
    lappend table_def {role "Role" {} {}}
}

lappend table_def {read_p "Read" {} "<td><a href=\"read-toggle.tcl?[export_url_vars user_id group_id role]&\[uplevel {set export_url_vars}\]\">[ad_html_pretty_boolean $read_p]</td>"}
lappend table_def {write_p "Write" {} "<td><a href=\"write-toggle.tcl?[export_url_vars user_id group_id role]&\[uplevel {set export_url_vars}\]\">[ad_html_pretty_boolean $write_p]</td>"}

if { [ad_parameter CommentPermissionsP gp] } {
    lappend table_def {comment_p "Comment" {} "<td><a href=\"comment-toggle.tcl?[export_url_vars user_id group_id role]&\[uplevel {set export_url_vars}\]\">[ad_html_pretty_boolean $comment_p]</td>"}
}

lappend table_def {owner_p "Owner" {} "<td><a href=\"owner-toggle.tcl?[export_url_vars user_id group_id role]&\[uplevel {set export_url_vars}\]\">[ad_html_pretty_boolean $owner_p]</td>"}

lappend table_def {actions "" {} "<td align=center><a href=\"remove-ug.tcl?[export_url_vars user_id group_id role]&\[uplevel {set export_url_vars}\]\">remove</a>"}

# public permissions
set extra_row_html "
<td>Public</td>\n
<td></td>\n"

set selection [ns_db 1row $db "select public_read_p, public_write_p, public_comment_p 
                               from general_permissions
                               where permissions_id = $permissions_id"]
set_variables_after_query

append extra_row_html "<td><a href=\"public-read-toggle.tcl?[export_url_vars permissions_id page_url page_name return_url return_name]\">[ad_html_pretty_boolean $public_read_p]</a></td>"

append extra_row_html "<td><a href=\"public-write-toggle.tcl?[export_url_vars permissions_id page_url page_name return_url return_name]\">[ad_html_pretty_boolean $public_write_p]</a></td>"

if { [ad_parameter CommentPermissionsP gp] } {
    append extra_row_html "<td><a href=\"public-comment-toggle.tcl?[export_url_vars permissions_id page_url page_name return_url return_name]\">[ad_html_pretty_boolean $public_comment_p]</a></td>"
}

ReturnHeaders

ns_write "[ad_header "View/Edit Permissions"]
<h2>View/Edit Permissions</h2>

$navbar 
<hr>
Permissions for $page_url_info
<p>
"

# spit out the dimensional bar
ns_write "[ad_dimensional $dimensional_list]<p>\n"

if { [ad_parameter GroupPermissionsP gp] } {
#     set sql "select gp.*, pum.*, users.*, user_groups.*
#              from general_permissions gp, permissions_ug_map pum, users, user_groups
#              where pum.group_id = user_groups.group_id(+)
#              and pum.user_id = users.user_id(+)
#              and gp.permissions_id = pum.permissions_id
#              and gp.permissions_id = $permissions_id
#              [ad_dimensional_sql $dimensional_list where]
#              [ad_order_by_from_sort_spec $orderby $table_def]"

    set sql "select gp.*, pum.*, users.*, user_groups.*
             from general_permissions gp, permissions_ug_map pum, users_view, user_groups_view
             where pum.group_id = user_groups.group_id
             and pum.user_id = users.user_id
             and gp.permissions_id = pum.permissions_id
             and gp.permissions_id = $permissions_id
             [ad_dimensional_sql $dimensional_list where]
             [ad_order_by_from_sort_spec $orderby $table_def]
             union
             select gp.*, pum.*, users.*, user_groups.*
             from general_permissions gp, permissions_ug_map pum, users_null users, user_groups_view
             where pum.group_id = user_groups.group_id
             and not exists (select 1 from users 
                             where user_id = pum.user_id)
             and gp.permissions_id = pum.permissions_id
             and gp.permissions_id = $permissions_id
             [ad_dimensional_sql $dimensional_list where]
             [ad_order_by_from_sort_spec $orderby $table_def]
             union
             select gp.*, pum.*, users.*, user_groups.*
             from general_permissions gp, permissions_ug_map pum, users, user_groups_null user_groups_view
             where not exists (select 1 from user_groups 
                               where group_id = pum.group_id)
             and pum.user_id = users.user_id
             and gp.permissions_id = pum.permissions_id
             and gp.permissions_id = $permissions_id
             [ad_dimensional_sql $dimensional_list where]
             [ad_order_by_from_sort_spec $orderby $table_def]
             union
             select gp.*, pum.*, users.*, user_groups.*
             from general_permissions gp, permissions_ug_map pum, users_null, user_groups_null
             where not exists (select 1 from user_groups
                               where group_id = pum.group_id)
             and not exists (select 1 from users 
                             where user_id = pum.user_id)
             and gp.permissions_id = pum.permissions_id
             and gp.permissions_id = $permissions_id
             [ad_dimensional_sql $dimensional_list where]
             [ad_order_by_from_sort_spec $orderby $table_def]"

} else {

#     set sql "select gp.*, pum.*, users.*, user_groups.*
#              from general_permissions gp, permissions_ug_map pum, users, user_groups
#              where pum.group_id = user_groups.group_id(+)
#              and pum.user_id = users.user_id(+)
#              and gp.permissions_id = pum.permissions_id
#              and gp.permissions_id = $permissions_id
#              and pum.group_id is null
#              [ad_dimensional_sql $dimensional_list where]
#              [ad_order_by_from_sort_spec $orderby $table_def]"

    set sql "select gp.*, pum.*, users.*, user_groups.*
             from general_permissions gp, permissions_ug_map pum, users_view, user_groups_view
             where pum.group_id = user_groups.group_id
             and pum.user_id = users.user_id
             and gp.permissions_id = pum.permissions_id
             and gp.permissions_id = $permissions_id
             and pum.group_id is null
             [ad_dimensional_sql $dimensional_list where]
             [ad_order_by_from_sort_spec $orderby $table_def]
             union
             select gp.*, pum.*, users.*, user_groups.*
             from general_permissions gp, permissions_ug_map pum, users_null users, user_groups_view
             where pum.group_id = user_groups.group_id
             and not exists (select 1 from users 
                             where user_id = pum.user_id)
             and gp.permissions_id = pum.permissions_id
             and gp.permissions_id = $permissions_id
             and pum.group_id is null
             [ad_dimensional_sql $dimensional_list where]
             [ad_order_by_from_sort_spec $orderby $table_def]
             union
             select gp.*, pum.*, users.*, user_groups.*
             from general_permissions gp, permissions_ug_map pum, users, user_groups_null user_groups_view
             where not exists (select 1 from user_groups 
                               where group_id = pum.group_id)
             and pum.user_id = users.user_id
             and gp.permissions_id = pum.permissions_id
             and gp.permissions_id = $permissions_id
             and pum.group_id is null
             [ad_dimensional_sql $dimensional_list where]
             [ad_order_by_from_sort_spec $orderby $table_def]
             union
             select gp.*, pum.*, users.*, user_groups.*
             from general_permissions gp, permissions_ug_map pum, users_null users, user_groups_null user_groups_view
             where not exists (select 1 from user_groups 
                               where group_id = pum.group_id)
             and not exists (select 1 from users 
                             where user_id = pum.user_id)
             and gp.permissions_id = pum.permissions_id
             and gp.permissions_id = $permissions_id
             and pum.group_id is null
             [ad_dimensional_sql $dimensional_list where]
             [ad_order_by_from_sort_spec $orderby $table_def]

"
}

set selection [ns_db select $db $sql] 

ns_write "
<blockquote>
[ad_table -Torderby $orderby -Textra_row_html $extra_row_html $db $selection $table_def]

<p>

</blockquote>
"

set selection [ns_db 1row $db "select count(*) as users_to_add
                               from users
                               where not exists (select 1 from permissions_ug_map pum
                                                 where permissions_id = $permissions_id
                                                 and users.user_id = pum.user_id)"]

set_variables_after_query

if { $users_to_add > 0 } {
    ns_write "
    <ul>
    <li><a href=\"user-add.tcl?[export_url_vars permissions_id page_url page_name return_url]\">Add a user</a></li>
    "
}

if { [ad_parameter GroupPermissionsP gp] } {

    set selection [ns_db 1row $db "select count(*) as groups_to_add
    from user_groups ug, user_group_types ugt
    where ug.group_type = ugt.group_type
    and existence_public_p = 't'
    and approved_p = 't'
    and exists 
    ((select role from user_group_roles ugr where ugr.group_id = ug.group_id union select null from dual)
    EXCEPT
    (select role from permissions_ug_map pum where pum.group_id = ug.group_id and permissions_id = $permissions_id))"]
    
    set_variables_after_query

    if { $groups_to_add > 0 } {
	ns_write "<li><a href=\"group-add.tcl?[export_url_vars permissions_id page_url page_name return_url]\">Add a group</a></li>"
    }
}

ns_write "</ul>"

if { [ad_parameter ShowFinishButtonP gp] } {
    ns_write "<form action=redirect-to-caller.tcl method=post>
    [export_form_vars page_url return_url]
    <center><input type=submit value=\"Finished editing permissions\"></center>
    </form>
"
}

ns_write "

[ad_footer]
"


